@@ -27,10 +27,13 @@
 ## -- 2022-01-21  2.2.0     SY       Add cycle_limit as an input parameter
 ## -- 2022-01-24  2.2.1     SY       Update seeding procedure, refactoring _reset()
 ## -- 2022-02-25  2.2.2     SY       Refactoring due to auto generated ID in class Dimension
+## -- 2022-05-23  2.2.3     SY       Bug fixing: Reward computation
+## -- 2022-05-30  2.2.4     SY       Replace 'energy' related parameters to 'power'
+## -- 2022-06-14  2.2.5     SY       Add termination condition for batch production scenario
 ## -------------------------------------------------------------------------------------------------
 
 """
-Ver. 2.2.2 (2022-02-25)
+Ver. 2.2.5 (2022-06-14)
 
 This module provides an RL environment of Bulk Good Laboratory Plant (BGLP).
 """
@@ -234,7 +237,7 @@ class VacuumPump (Actuator):
         
 
 ## -------------------------------------------------------------------------------------------------
-    def calc_energy(self):
+    def calc_power(self):
         """
         This method calculates the power consumption of a vacuum pump.
 
@@ -290,32 +293,32 @@ class Belt(Actuator):
     Parameters
     ----------
     name : str
-        specific name or id of a vacuum pump (e.g. Belt_A, etc.).
+        specific name or id of an actuator (e.g. Belt_A, etc.).
     actiontype : str
         "C" for continuous action, "B" for binary action.
     minpower : float
-        minimum power of a vacuum pump.
+        minimum power of an actuator.
     maxpower : float
-        maximum power of a vacuum pump.
+        maximum power of an actuator.
     minaction : float
-        minimum action of a vacuum pump.
+        minimum action of an actuator.
     maxaction : float
-        maximum action of a vacuum pump.
+        maximum action of an actuator.
     masscoeff : float
-        mass transport coefficient of a vacuum pump.
+        mass transport coefficient of an actuator.
         
     Attributes
     ----------
     reg_b : list of objects
-        list of existing belts.
+        list of existing actuators.
     idx_b : int
         length of reg_b.
     name : str
-        specific name or id of a belt.
+        specific name or id of an actuator.
     actiontype : str
         "C" for continuous action, "B" for binary action.
     speed : float
-        speed of a bel.
+        speed of an actuator.
     """
     reg_b       = []
     idx_b       = 0
@@ -392,7 +395,7 @@ class Belt(Actuator):
     
 
 ## -------------------------------------------------------------------------------------------------
-    def calc_energy(self):
+    def calc_power(self):
         """
         This method calculates the power consumption of a belt.
 
@@ -639,8 +642,8 @@ class BGLP (Environment):
         the learning rate for margin parameter, related to implemented reward function. The default is 1.0.
     lr_demand : float, optional
         the learning rate for production demand, related to implemented reward function. The default is 4.0.
-    lr_energy : float, optional
-        the learning rate for energy consumption, related to implemented reward function. The default is 0.0010.
+    lr_power : float, optional
+        the learning rate for power consumption, related to implemented reward function. The default is 0.0010.
     margin_p : list of floats, optional
         the margin parameter of reservoirs [low, high, multplicator]. The default is [0.2,0.8,4].
     prod_target : float, optional
@@ -684,11 +687,11 @@ class BGLP (Environment):
         the learning rate for margin parameter.
     lr_demand : float
         the learning rate for production demand.
-    lr_energy : float
-        the learning rate for energy consumption.
+    lr_power : float
+        the learning rate for power consumption.
     overflow : list of floats
         current overflow.
-    energy : list of floats
+    power : list of floats
         current power consumption.
     transport : list of floats
         current transported mass flow.
@@ -700,7 +703,7 @@ class BGLP (Environment):
         current overflow for a specific buffer in a specific time.
     demand_t : float
         current demand for a specific buffer in a specific time.
-    energy_t : float
+    power_t : float
         current power consumption for a specific actuator in a specific time.
     transport_t : float
         current transported material by a specific actuaor in a specific time.
@@ -734,17 +737,17 @@ class BGLP (Environment):
     t_step              = 0
     lr_margin           = 0
     lr_demand           = 0
-    lr_energy           = 0
+    lr_power            = 0
     overflow            = []
     demand              = []
-    energy              = []
+    power               = []
     transport           = []
     reward              = []
     levels_init         = 0
     reset_levels        = 0
     overflow_t          = 0
     demand_t            = 0
-    energy_t            = 0
+    power_t             = 0
     transport_t         = 0
     margin_t            = 0
     prod_reached        = 0
@@ -754,7 +757,7 @@ class BGLP (Environment):
 ## -------------------------------------------------------------------------------------------------
     def __init__(self, p_reward_type=Reward.C_TYPE_EVERY_AGENT, p_logging=Log.C_LOG_ALL,
                  t_step=0.5, t_set=10.0, demand=0.1, lr_margin=1.0, lr_demand=4.0,
-                 lr_energy=0.0010, margin_p=[0.2,0.8,4], prod_target=10000,
+                 lr_power=0.0010, margin_p=[0.2,0.8,4], prod_target=10000,
                  prod_scenario='continuous', cycle_limit=0):
         self.num_envs       = 5                                                 # Number of internal sub-environments
         self.reward_type    = p_reward_type
@@ -777,7 +780,7 @@ class BGLP (Environment):
         self._demand        = demand
         self.lr_margin      = lr_margin
         self.lr_demand      = lr_demand
-        self.lr_energy      = lr_energy
+        self.lr_power       = lr_power
         self.prod_target    = prod_target
         self.prod_scenario  = prod_scenario
         self.levels_init    = np.ones((6,1))*0.5
@@ -829,17 +832,17 @@ class BGLP (Environment):
         self.margin             = np.zeros((len(self.ress),1))
         self.overflow           = np.zeros((len(self.ress),1))
         self.demand             = np.zeros((len(self.ress),1))
-        self.energy             = np.zeros((len(self.acts),1))
+        self.power              = np.zeros((len(self.acts),1))
         self.transport          = np.zeros((len(self.acts),1))
         self.overflow_t         = np.zeros((len(self.ress),1))
         self.demand_t           = np.zeros((len(self.ress),1))
-        self.energy_t           = np.zeros((len(self.acts),1))
+        self.power_t            = np.zeros((len(self.acts),1))
         self.transport_t        = np.zeros((len(self.acts),1))
         self.margin_t           = np.zeros((len(self.ress),1))
         self.reward             = np.zeros((len(self.acts),1))
         self.con_res_to_act     = [[-1,0],[0,1],[1,2],[2,3],[3,4],[4,-1]]
         
-        self.data_lists         = ["time","overflow","energy","demand"]
+        self.data_lists         = ["time","overflow","power","demand"]
         self.data_storing       = DataStoring(self.data_lists)
         self.data_frame         = None
         
@@ -952,16 +955,16 @@ class BGLP (Environment):
         
         self.overflow_t         = np.zeros((len(self.ress),1))
         self.demand_t           = np.zeros((len(self.ress),1))
-        self.energy_t           = np.zeros((len(self.acts),1))
+        self.power_t            = np.zeros((len(self.acts),1))
         self.transport_t        = np.zeros((len(self.acts),1))
         self.margin_t           = np.zeros((len(self.ress),1))
         
         x = 0
         while x < (self.t_set//self.t_step):
-            overflow_diff, demand_diff, energy_diff, transport_diff, margin_diff = self.get_status(self.t, self._demand)
+            overflow_diff, demand_diff, power_diff, transport_diff, margin_diff = self.get_status(self.t, self._demand)
             self.overflow_t     += overflow_diff
             self.demand_t       += demand_diff
-            self.energy_t       += energy_diff
+            self.power_t       += power_diff
             self.transport_t    += transport_diff
             self.margin_t       += margin_diff
             self.t              += self.t_step
@@ -974,7 +977,7 @@ class BGLP (Environment):
         
         self.data_storing.memorize("time",str(self.data_frame),self.t)
         self.data_storing.memorize("overflow",str(self.data_frame), (sum(self.overflow_t[:])/self.t_set).item())
-        self.data_storing.memorize("energy",str(self.data_frame), (sum(self.energy_t[:])/self.t_set).item())
+        self.data_storing.memorize("power",str(self.data_frame), (sum(self.power_t[:])/self.t_set).item())
         self.data_storing.memorize("demand",str(self.data_frame), (self.demand_t[-1]/self.t_set).item())
 
         return self._state
@@ -1000,6 +1003,7 @@ class BGLP (Environment):
             return False
         else:
             if self.prod_reached >= self.prod_target:
+                self._state.set_terminal(True)
                 return True
             else:
                 return False
@@ -1048,24 +1052,24 @@ class BGLP (Environment):
             r_overall = 0
             r_overall = r_overall + sum(self.calc_reward()).item()
             reward.set_overall_reward(r_overall)
-
+        
+        elif self.reward_type == Reward.C_TYPE_EVERY_AGENT:
+           for agent_id in self._last_action.get_agent_ids():
+               r_reward = self.calc_reward()
+               reward.add_agent_reward(agent_id, r_reward[int(agent_id)])
+               
         else:
            for agent_id in self._last_action.get_agent_ids():
-                agent_action_elem   = self._last_action.get_elem(agent_id)
-                agent_action_ids    = agent_action_elem.get_dim_ids()
-                r_agent             = 0
-                r_reward            = self.calc_reward()
-                action_idx          = 0
+                agent_action_elem = self._last_action.get_elem(agent_id)
+                agent_action_ids = agent_action_elem.get_dim_ids()
+                r_agent = 0
+                r_reward = self.calc_reward()
+                action_idx = 0
                 for action_id in agent_action_ids:
-                    r_action        = r_reward[action_idx]
-                    action_idx      += 1
-                    if self.reward_type == Reward.C_TYPE_EVERY_ACTION:
-                        reward.add_action_reward(agent_id, action_id, r_action)
-                    elif self.reward_type == Reward.C_TYPE_EVERY_AGENT:
-                        r_agent = r_agent + r_action
-                if self.reward_type == Reward.C_TYPE_EVERY_AGENT:
-                    r_agent = r_agent / len(agent_action_ids)
-                    reward.add_agent_reward(agent_id, r_agent)
+                    r_action = r_reward[action_idx]
+                    action_idx += 1
+                    reward.add_action_reward(agent_id, action_id, r_action)
+                    
         return reward
 
   
@@ -1080,13 +1084,13 @@ class BGLP (Environment):
 
 
 ## -------------------------------------------------------------------------------------------------
-    def calc_energy(self):
+    def calc_power(self):
         """
         This method calculates the power consumptions per actuator.
         
         """
         for act_num in range(len(self.acts)):
-            self.energy[act_num] = self.acts[act_num].calc_energy()*self.t_step
+            self.power[act_num] = self.acts[act_num].calc_power()*self.t_step
                 
 
 ## -------------------------------------------------------------------------------------------------
@@ -1207,7 +1211,7 @@ class BGLP (Environment):
 ## -------------------------------------------------------------------------------------------------
     def get_status(self, t, demandval):
         """
-        This method calculates overflow, demand, energy, transport, and margin.
+        This method calculates overflow, demand, power, transport, and margin.
         This function will be called every time step.
         
         Parameters
@@ -1223,7 +1227,7 @@ class BGLP (Environment):
             overflow levels.
         demand : list of floats
             demand fulfilled.
-        energy : list of floats
+        power : list of floats
             power consumptions.
         transport : list of floats
             transported materials.
@@ -1233,10 +1237,10 @@ class BGLP (Environment):
         """
         self.t = t       
         self.calc_mass_flows()
-        self.calc_energy()
+        self.calc_power()
         self.calc_margin()
         self.update(demandval)        
-        return self.overflow, self.demand, self.energy, self.transport, self.margin
+        return self.overflow, self.demand, self.power, self.transport, self.margin
             
 
 ## -------------------------------------------------------------------------------------------------
@@ -1290,7 +1294,7 @@ class BGLP (Environment):
         """
         for actnum in range(len(self.acts)):
             acts = self.acts[actnum]
-            self.reward[actnum] = 1/(1+self.lr_margin*self.margin_t[actnum])+1/(1+self.lr_energy*self.energy_t[actnum]/(acts.power_max/1000.0))
+            self.reward[actnum] = 1/(1+self.lr_margin*self.margin_t[actnum])+1/(1+self.lr_power*self.power_t[actnum]/(acts.power_max/1000.0))
             if actnum == len(self.acts)-1:
                 self.reward[actnum] += 1/(1-self.lr_demand*self.demand_t[-1])
             else:
